{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
osCommerce Product Manager for Windows (oscpmwin).
Copyright 2003,2004,2005 by Mario A. Valdez-Ramirez.

You can contact Mario A. Valdez-Ramirez
by email at mario@mariovaldez.org or paper mail at
Olmos 809, San Nicolas, NL. 66495, Mexico.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
unit balloons;

interface

USES Windows, Forms;

CONST
  balloonC_Def_BalloonWidth = 250;
  balloonC_Def_NormalHintWidth = 40;
  balloonC_Def_BalloonDelay = 30000;
  balloonC_Def_BalloonIni = 500;
  balloonC_Balloon_Separator = '|';
  balloonC_MaxBalloonChars = 640;
  balloonC_MaxBallons = 1000;


TYPE
  balloonR_ToolTip = RECORD
                    TTHandle : DWORD;
                    CtHandle : DWORD;
                  END;
  balloonT_ToolTipList = ARRAY [0..balloonC_MaxBallons] OF balloonR_ToolTip;

  FUNCTION FNballoon_CreateBalloon (BalloonTitle, BalloonText : STRING; BalloonType : WORD; ControlHandle : DWORD; CurForm : TForm) : DWORD;
  FUNCTION FNballoon_DestroyBalloon (ControlHandle : DWORD) : DWORD;
  FUNCTION FNballoon_GetBalloonHandle (ControlHandle : DWORD; ClearHandles : BOOLEAN) : DWORD;
  FUNCTION FNballoon_UpdateBalloon (BalloonText : STRING; BalloonType : WORD; ControlHandle : DWORD; CurForm : TForm) : DWORD;
  PROCEDURE PRballoon_DestroyAllBalloons;
  PROCEDURE PRballoon_ResetBalloons;
  PROCEDURE PRballoon_SetFormBalloons (CurForm : TForm; AllowBalloons : INTEGER);
  PROCEDURE PRballoon_ResetBalloonsType (UseNormalHints : BOOLEAN);


VAR
  balloon_HandleList : balloonT_ToolTipList;
  balloon_TotalBalloons : WORD;
  balloon_UseNormalTootips : BOOLEAN;


implementation

USES Commctrl, Controls, SysUtils, gnugettext;

{http://msdn.microsoft.com/library/en-us/shellcc/platform/commctls/tooltip/usingtooltips.asp}
{http://msdn.microsoft.com/library/en-us/shellcc/platform/commctls/tooltip/structures/toolinfo.asp}
{http://msdn.microsoft.com/library/en-us/shellcc/platform/commctls/tooltip/reflist.asp}



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{0 No icon. 1 Info icon. 2 Warning icon. 3 Error Icon.}
FUNCTION FNballoon_CreateBalloon (BalloonTitle, BalloonText : STRING;
                                  BalloonType : WORD; ControlHandle : DWORD;
                                  CurForm : TForm) : DWORD;
CONST
  opm_TTS_BALLOON = $0040;
  opm_TTM_SETTITLE = $0420;
VAR
  BalloonToolTip : THandle;
  ToolTipData : TOOLINFO;
BEGIN
  BalloonToolTip := CreateWindow (TOOLTIPS_CLASS, NIL, WS_POPUP OR TTS_NOPREFIX OR opm_TTS_BALLOON OR TTS_ALWAYSTIP, 0, 0, 0, 0, ControlHandle, 0, Application.Handle, NIL);
  ToolTipData.cbSize := SIZEOF (TOOLINFO);
  ToolTipData.uFlags := (TTF_SUBCLASS OR TTF_DI_SETITEM OR TTF_TRANSPARENT);
  ToolTipData.hwnd := ControlHandle;
  ToolTipData.hinst := Application.Handle;
  ToolTipData.uId := 0;
  ToolTipData.lpszText := PCHAR (BalloonText);
  ToolTipData.rect := CurForm.ClientRect;
  SendMessage (BalloonToolTip, TTM_ADDTOOL, 0, INTEGER (@ToolTipData));
  SendMessage (BalloonToolTip, opm_TTM_SETTITLE, BalloonType, Integer (PCHAR (BalloonTitle)));
  SendMessage (BalloonToolTip, TTM_SETMAXTIPWIDTH, 0, balloonC_Def_BalloonWidth);
  SendMessage (BalloonToolTip, TTM_SETDELAYTIME, TTDT_AUTOPOP, balloonC_Def_BalloonDelay);
  SendMessage (BalloonToolTip, TTM_SETDELAYTIME, TTDT_INITIAL, balloonC_Def_BalloonIni);
  SendMessage (BalloonToolTip, TTM_SETDELAYTIME, TTDT_RESHOW, balloonC_Def_BalloonIni);
  FNballoon_CreateBalloon := BalloonToolTip;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNballoon_GetBalloonHandle (ControlHandle : DWORD; ClearHandles : BOOLEAN) : DWORD;
VAR
  CurTT : LONGINT;
BEGIN
  FNballoon_GetBalloonHandle := 0;
  IF (balloon_TotalBalloons > 0) THEN
    FOR CurTT := 0 TO (balloon_TotalBalloons - 1) DO
      BEGIN
        IF (balloon_HandleList[CurTT].CtHandle = ControlHandle) THEN
          BEGIN
            FNballoon_GetBalloonHandle := balloon_HandleList[CurTT].TTHandle;
            IF (ClearHandles = TRUE) THEN
              BEGIN
                balloon_HandleList[CurTT].TTHandle := 0;
                balloon_HandleList[CurTT].CtHandle := 0;
              END;
            BREAK;
          END;
      END;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRballoon_DestroyAllBalloons;
VAR
  ToolTipData : TOOLINFO;
  CurTT : WORD;
BEGIN
  IF (balloon_TotalBalloons > 0) THEN
    FOR CurTT := 0 TO (balloon_TotalBalloons - 1) DO
      BEGIN
        IF (balloon_HandleList[CurTT].CtHandle <> 0) THEN
          BEGIN
            ToolTipData.cbSize := SIZEOF (TOOLINFO);
            ToolTipData.hwnd := balloon_HandleList[CurTT].CtHandle;
            ToolTipData.uId := 0;
            SendMessage (balloon_HandleList[CurTT].TTHandle, TTM_DELTOOL, 0, INTEGER (@ToolTipData));
            DestroyWindow (balloon_HandleList[CurTT].TTHandle);
            balloon_HandleList[CurTT].TTHandle := 0;
            balloon_HandleList[CurTT].CtHandle := 0;
          END;
      END;
  balloon_TotalBalloons := 0;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNballoon_DestroyBalloon (ControlHandle : DWORD) : DWORD;
VAR
  BalloonToolTip : THandle;
  ToolTipData : TOOLINFO;
BEGIN
  BalloonToolTip := FNballoon_GetBalloonHandle (ControlHandle, TRUE);
  IF (BalloonToolTip <> 0) THEN
    BEGIN
      ToolTipData.cbSize := SIZEOF (TOOLINFO);
      ToolTipData.hwnd := ControlHandle;
      ToolTipData.uId := 0;
      SendMessage (BalloonToolTip, TTM_DELTOOL, 0, INTEGER (@ToolTipData));
      DestroyWindow (BalloonToolTip);
      FNballoon_DestroyBalloon := BalloonToolTip;
    END
  ELSE FNballoon_DestroyBalloon := 0;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRballoon_ResetBalloons;
BEGIN
  balloon_TotalBalloons := 0;
  Application.ShowHint := balloon_UseNormalTootips;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRballoon_ResetBalloonsType (UseNormalHints : BOOLEAN);
BEGIN
  balloon_UseNormalTootips := UseNormalHints;
  Application.ShowHint := UseNormalHints;
  Application.HintPause := balloonC_Def_BalloonIni;
  Application.HintHidePause := balloonC_Def_BalloonDelay;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION FNballoon_UpdateBalloon (BalloonText : STRING;
                                  BalloonType : WORD; ControlHandle : DWORD;
                                  CurForm : TForm) : DWORD;
CONST
  opm_TTM_SETTITLE = $0420;
VAR
  BalloonToolTip : THandle;
  ToolTipData : TOOLINFO;
  TitleMsg : STRING;
BEGIN
  TitleMsg := '';
  IF (POS (balloonC_Balloon_Separator, BalloonText) > 0) THEN
    BEGIN
      TitleMsg := TRIM (COPY (BalloonText, 1, (POS (balloonC_Balloon_Separator, BalloonText) - 1)));
      BalloonText := TRIM (COPY (BalloonText, (POS (balloonC_Balloon_Separator, BalloonText) + 1), balloonC_MaxBalloonChars));
    END
  ELSE BalloonText := TRIM (COPY (BalloonText, 1, balloonC_MaxBalloonChars));
  BalloonToolTip := FNballoon_GetBalloonHandle (ControlHandle, FALSE);
  IF (BalloonToolTip <> 0) THEN
    BEGIN
      ToolTipData.cbSize := SIZEOF (TOOLINFO);
      ToolTipData.uFlags := (TTF_SUBCLASS OR TTF_DI_SETITEM OR TTF_TRANSPARENT);
      ToolTipData.hwnd := ControlHandle;
      ToolTipData.hinst := Application.Handle;
      ToolTipData.uId := 0;
      ToolTipData.lpszText := PCHAR (BalloonText);
      ToolTipData.rect := CurForm.ClientRect;
      SendMessage (BalloonToolTip, TTM_UPDATETIPTEXT, 0, INTEGER (@ToolTipData));
      SendMessage (BalloonToolTip, opm_TTM_SETTITLE, BalloonType, Integer (PCHAR (TitleMsg)));
      FNballoon_UpdateBalloon := BalloonToolTip;
    END
  ELSE FNballoon_UpdateBalloon := 0;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE PRballoon_SetFormBalloons (CurForm : TForm; AllowBalloons : INTEGER);
VAR
  CurCtl : LONGINT;
  TitleMsg, BodyMsg : STRING;
BEGIN
  IF ((AllowBalloons > 0) AND (CurForm.ComponentCount > 0)) THEN
    FOR CurCtl := 0 TO (CurForm.ComponentCount - 1) DO
      IF CurForm.Components[CurCtl] IS TWinControl THEN
        WITH CurForm.Components[CurCtl] AS TWinControl DO
          BEGIN
            IF (balloon_UseNormalTootips = FALSE) THEN
              BEGIN
                IF ((Hint <> '') AND (balloon_TotalBalloons < balloonC_MaxBallons)) THEN
                  BEGIN
                    TitleMsg := '';
                    BodyMsg := ANSISTRING (_(Hint));
                    IF (POS (balloonC_Balloon_Separator, BodyMsg) > 0) THEN
                      BEGIN
                        TitleMsg := TRIM (COPY (BodyMsg, 1, (POS (balloonC_Balloon_Separator, BodyMsg) - 1)));
                        BodyMsg := TRIM (COPY (BodyMsg, (POS (balloonC_Balloon_Separator, BodyMsg) + 1), balloonC_MaxBalloonChars));
                      END
                    ELSE BodyMsg := TRIM (COPY (BodyMsg, 1, balloonC_MaxBalloonChars));
                    balloon_HandleList[balloon_TotalBalloons].TTHandle := FNballoon_CreateBalloon (TitleMsg, PCHAR(BodyMsg), 1, Handle, CurForm);
                    balloon_HandleList[balloon_TotalBalloons].CtHandle := Handle;
                    INC (balloon_TotalBalloons);
                  END;
                ShowHint := FALSE;
              END
            ELSE
              BEGIN
                IF (Hint <> '') THEN
                  BEGIN
                    ShowHint := TRUE;
                    BodyMsg := ANSISTRING (_(Hint));
                    IF (POS (balloonC_Balloon_Separator, BodyMsg) > 0) THEN
                      BEGIN
                        Hint := TRIM (COPY (BodyMsg, 1, (POS (balloonC_Balloon_Separator, BodyMsg) - 1))) + ':' + #13#10#13#10;
                        Hint := Hint + WRAPTEXT (TRIM (COPY (BodyMsg, (POS (balloonC_Balloon_Separator, BodyMsg) + 1), balloonC_MaxBalloonChars)), #13#10, [' '], balloonC_Def_NormalHintWidth);
                      END
                    ELSE
                      Hint := WRAPTEXT (TRIM (COPY (BodyMsg, 1, balloonC_MaxBalloonChars)), #13#10, [' '], balloonC_Def_NormalHintWidth);
                  END
                ELSE ShowHint := FALSE;
              END;
          END;
END;




INITIALIZATION

  balloon_UseNormalTootips := FALSE;
  balloon_TotalBalloons := 0;
  Application.ShowHint := balloon_UseNormalTootips;





end.
